@@ -6,6 +6,7 @@ import pkgutil
 
 import dcyd.utils.constants as constants
 
+
 def args_parser():
     parser = argparse.ArgumentParser(description='Configure client to communicate with dcyd.')
     parser.add_argument(
@@ -33,8 +34,10 @@ def args_parser():
 
 
 def configure_client(*, project_id, project_access_token, config_file):
-    if not project_id: raise ValueError('project_id is required')
-    if not project_access_token: raise ValueError('project_access_token is required')
+    if not project_id:
+        raise ValueError('project_id is required')
+    if not project_access_token:
+        raise ValueError('project_access_token is required')
 
     config_json = json.dumps({
         'project_id': project_id,
@@ -65,4 +68,3 @@ def main():
         print(f"As a final step, run `export DCYD_CONFIG_FILE={config_file}`.")
     else:
         parser.print_help()
-
