@@ -2,28 +2,49 @@
 
 import base64
 import json
+import logging
 import os
 import pathlib
 import pickle
+import random
 
 import dcyd
 from . import constants
 
-def get_project_id():
+
+def generate_uid() -> str:
+    '''
+    Generate a sha1-like random hex string, 10 times faster than uuid.uuid4()
+
+    The below illustrate the performance of similar solutions
+    ```
+    print(timeit.timeit(stmt=lambda: uuid.uuid4()))
+    print(timeit.timeit(stmt=lambda: secrets.token_hex(16)))
+    print(timeit.timeit(stmt=lambda: hashlib.sha1(random.getrandbits(64).to_bytes(8, 'big')).hexdigest()))
+    print(timeit.timeit(stmt=lambda: random.getrandbits(160).to_bytes(20, 'big').hex()))
+    ```
+    '''
+    return random.getrandbits(160).to_bytes(20, 'big').hex()
+
+
+def get_project_id() -> str:
     '''Returns project_id from either env var or config file'''
 
     project_id = get_project_credentials()['project_id']
 
     if not project_id:
-        raise ValueError("Please set either DCYD_PROJECT_ID or DCYD_CONFIG_FILE env var")
+        report_client_error("Please set either DCYD_PROJECT_ID or DCYD_CONFIG_FILE env var")
+        return ''
 
     return project_id
 
+
 CONFIG_FROM_FILE = {}
 if os.getenv('DCYD_CONFIG_FILE'):
     CONFIG_FROM_FILE = json.loads(pathlib.Path(os.getenv('DCYD_CONFIG_FILE')).read_text())
 
-def get_project_credentials():
+
+def get_project_credentials() -> dict:
     '''
     Returns project_access_token from either env var or config file
 
@@ -41,16 +62,20 @@ def get_project_credentials():
         'project_access_token': project_access_token,
     }
 
-def serialize_objects(obj_dict):
-    errors = []
-    serialized_data = {}
-    for k,v in obj_dict.items():
-        serialized_data[k], success = make_json_serializable(v)
-        if not success:
-            errors.append(k)
-    return serialized_data, errors
 
-def make_json_serializable(obj):
+CLIENT_ERROR_MODE = os.getenv('DCYD_CLIENT_ERROR_MODE', 'log')
+
+
+def report_client_error(message: str):
+    if CLIENT_ERROR_MODE == 'raise_error':
+        raise RuntimeError(message)
+    elif CLIENT_ERROR_MODE == 'silent':
+        return
+    else:
+        logging.error(message)
+
+
+def base64_pickle(obj):
     '''To ensure obj is JSON-serializable, pickle and base64-encode it.
 
     arg obj: object to be tested for JSON-serializability
@@ -59,16 +84,17 @@ def make_json_serializable(obj):
     returns: a transformation of obj that is JSON-serializable.
     '''
     try:
-        encoded_obj = base64.b64encode( # encode byte string into bytes
-                        pickle.dumps(obj) # convert object into a byte string
-                      ).decode('ascii')
+        encoded_obj = base64.b64encode(  # encode byte string into bytes
+            pickle.dumps(obj)  # convert object into a byte string
+        ).decode('ascii')
         success = True
     except Exception as e:
         encoded_obj = base64.b64encode(
-                        pickle.dumps(e)
-                      ).decode('ascii')
+            pickle.dumps(e)
+        ).decode('ascii')
         success = False
     return encoded_obj, success
 
-def api_url(path=''):
+
+def api_url(path='') -> str:
     return os.getenv('DCYD_API_BASE', 'https://api.dcyd.io') + '/' + path
